(** Pure literal elimination.
 *
 * The pure rule consists in eliminating a literal when its opposite does not occur in the schema/block.
 * ex. P_n is pure in /\i=1..n ~P_i-1 \/ P_i  as ~P_n does not occur in the schema.
 * This is not the case of P_n-1 as ~P_n-1 occurs in it.
 *
 * We have to detect such literals (though this is usually an optimisation, it is here mandatory for looping to occur).
 * This is achieved through the following module.
 *
 * @author Vincent Aravantinos
 *)

(** The type of purities. 
 * The information to detect pure literals is collected independantly of the detection itself.
 * Hence the need to store it in some way, and so the need for a dedicated type.
 *)
type t

(** The interval of values possibly taken by a variable e.g. (i,1,n) for /\i=1..n S. *)
type range = Variable.t * Indexes.t * Indexes.t

(** The empty purity. *)
val empty : t

(** Adds a literal, possibly within a given range, to a purity
 * meaning that the returned purity considers this literal as occurring 
 * (thus its negation is not pure).
 *)
val add : ?range:range option -> lit:Literal.t -> t -> t

(** Merges two purities. *)
val merge : t -> t -> t

(** From a constraint and an already computed purity, says if a literal is pure.
 * Notice that the constraint is needed because P_1 is pure in ~P_n /\ ... if n <> 1, but is not if n may equal 1. *)
val is_pure : cstr:Constraint.t -> pur:t -> Literal.t -> bool

(** The interface of a data structure that can be inspected for pure literals. *)
module type Purable =
  sig
    (** The type of the data structure. *)
    type purable

    (** Updates a purity depending on an object of type [purable] *)
    val purity_add : purable -> ?range:range option -> pur:t -> t

    (** Takes an object of type [purable] and eliminates the pure literals inside according to [pur] and [cstr].
     * Result is tagged with {!Modified} information for the sake of efficiency.
     *)
    val purify : purable -> cstr:Constraint.t -> pur:t -> Modified.t * purable
  end
